package com.vieaura.plugins.rw_voice

import androidx.annotation.NonNull
import android.Manifest
import android.app.Activity
import android.content.Intent
import android.content.pm.PackageManager
import android.os.Bundle
import androidx.core.content.ContextCompat
import androidx.core.app.ActivityCompat

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding


class VoicePluginActivity : Activity() {
    private val DICTATION_REQUEST_CODE = 34
    // Dictation intent action
    private val ACTION_DICTATION = "com.realwear.keyboard.intent.action.DICTATION";

    private var numeric = false
    private var text = false


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        numeric = intent.getBooleanExtra("numeric", false)
        text = intent.getBooleanExtra("text", false)

        launchApplet()
 
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, intent: Intent?) {
        if (resultCode == RESULT_OK && intent != null) {
        
          RwVoicePlugin.onVoiceResult(intent?.getStringExtra("result"))

        } else {
            RwVoicePlugin.onEmptyResult()
        }

        finish()
    }

    private fun launchApplet() {
        val intent = Intent(ACTION_DICTATION) // check for numeric or text later 
        if (numeric) intent.putExtra("numeric", true);
        if (text) intent.putExtra("text", true);
        startActivityForResult(intent, DICTATION_REQUEST_CODE)
    }
}

/** RwVoicePlugin */
class RwVoicePlugin: FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel
  lateinit var activity: Activity

  companion object {
        private const val CHANNEL = "com.vieaura.plugins/rw_voice"
        private lateinit var methodResult: Result

        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val channel = MethodChannel(registrar.messenger(), CHANNEL)
            val plugin = RwVoicePlugin()

            plugin.activity = registrar.activity()
            channel.setMethodCallHandler(plugin)
        }

        @JvmStatic
        fun onVoiceResult(result: String?) {
            if (result != null) {
                methodResult.success(result)
            } else {
                methodResult.error(RwVoicePlugin::class.java.canonicalName, "Unable to capture voice", null)
            }
        }

        @JvmStatic
        fun onEmptyResult() {
            methodResult.success(null)
        }
    }

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, CHANNEL)
    //val channel = MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), CHANNEL)
    channel.setMethodCallHandler(this)
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
  
  /*     if (call.method == "getPlatformVersion") {
      result.success("Android ${android.os.Build.VERSION.RELEASE}")
    } else {
      result.notImplemented()
    } */

    when (call.method) {
      "voiceResult" -> {
          val intent = Intent(activity, VoicePluginActivity::class.java)

          //intent.putExtra("numeric", false)

          methodResult = result
          activity.startActivity(intent)
      }

      else ->
          result.notImplemented()
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
      this.activity = binding.activity
  }

  override fun onDetachedFromActivity() {
  }

  override fun onDetachedFromActivityForConfigChanges() {
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
  }
}
